@@ -1,11 +1,16 @@
-/* Copyright (c) 1993-2002
+/* Copyright (c) 2008, 2009
+ *      Juergen Weigert (jnweiger@immd4.informatik.uni-erlangen.de)
+ *      Michael Schroeder (mlschroe@immd4.informatik.uni-erlangen.de)
+ *      Micah Cowan (micah@cowan.name)
+ *      Sadrul Habib Chowdhury (sadrul@users.sourceforge.net)
+ * Copyright (c) 1993-2002, 2003, 2005, 2006, 2007
  *      Juergen Weigert (jnweiger@immd4.informatik.uni-erlangen.de)
  *      Michael Schroeder (mlschroe@immd4.informatik.uni-erlangen.de)
  * Copyright (c) 1987 Oliver Laumann
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
+ * the Free Software Foundation; either version 3, or (at your option)
  * any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -14,12 +19,12 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program (see the file COPYING); if not, write to the
- * Free Software Foundation, Inc.,
- * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
+ * along with this program (see the file COPYING); if not, see
+ * http://www.gnu.org/licenses/, or contact Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA  02111-1301  USA
  *
  ****************************************************************
- * $Id: image.h,v 1.9 1994/05/31 12:31:54 mlschroe Exp $ FAU
+ * $Id$ GNU
  */
 
 
@@ -28,8 +33,10 @@
 
 #ifdef FONT
 # define IFFONT(x) x
+# define IFFONTX(x) x
 #else
 # define IFFONT(x)
+# define IFFONTX(x)
 #endif
 
 #ifdef COLOR
@@ -53,18 +60,20 @@
 struct mchar {
 	 unsigned char image;
 	 unsigned char attr;
-IFFONT(  unsigned char font;)
-IFCOLOR( unsigned char color;)
-IFCOLORX(unsigned char colorx;)
-IFDWCHAR(unsigned char mbcs;)
+IFFONT(  unsigned char font; )
+IFFONTX( unsigned char fontx; )
+IFCOLOR( unsigned char color; )
+IFCOLORX(unsigned char colorx; )
+IFDWCHAR(unsigned char mbcs; )
 };
 
 struct mline {
 	 unsigned char *image;
 	 unsigned char *attr;
-IFFONT(  unsigned char *font;)
-IFCOLOR( unsigned char *color;)
-IFCOLORX(unsigned char *colorx;)
+IFFONT(  unsigned char *font; )
+IFFONTX( unsigned char *fontx; )
+IFCOLOR( unsigned char *color; )
+IFCOLORX(unsigned char *colorx; )
 };
 
 
@@ -72,66 +81,74 @@ IFCOLORX(unsigned char *colorx;)
 #define save_mline(ml, n) do {						\
 	 bcopy((char *)(ml)->image, (char *)mline_old.image, (n));	\
 	 bcopy((char *)(ml)->attr,  (char *)mline_old.attr,  (n));	\
-IFFONT(	 bcopy((char *)(ml)->font,  (char *)mline_old.font,  (n));     )\
-IFCOLOR( bcopy((char *)(ml)->color, (char *)mline_old.color, (n));     )\
-IFCOLORX(bcopy((char *)(ml)->colorx, (char *)mline_old.colorx, (n));   )\
+IFFONT(	 bcopy((char *)(ml)->font,  (char *)mline_old.font,  (n));    ) \
+IFFONTX( bcopy((char *)(ml)->fontx, (char *)mline_old.fontx, (n));    ) \
+IFCOLOR( bcopy((char *)(ml)->color, (char *)mline_old.color, (n));    ) \
+IFCOLORX(bcopy((char *)(ml)->colorx, (char *)mline_old.colorx, (n));  ) \
 } while (0)
 
 #define bcopy_mline(ml, xf, xt, n) do {					       \
 	 bcopy((char *)(ml)->image + (xf), (char *)(ml)->image + (xt), (n));   \
 	 bcopy((char *)(ml)->attr  + (xf), (char *)(ml)->attr  + (xt), (n));   \
-IFFONT(	 bcopy((char *)(ml)->font  + (xf), (char *)(ml)->font  + (xt), (n));  )\
-IFCOLOR( bcopy((char *)(ml)->color + (xf), (char *)(ml)->color + (xt), (n));  )\
-IFCOLORX(bcopy((char *)(ml)->colorx + (xf), (char *)(ml)->colorx + (xt), (n));)\
+IFFONT(	 bcopy((char *)(ml)->font  + (xf), (char *)(ml)->font  + (xt), (n)); ) \
+IFFONTX( bcopy((char *)(ml)->fontx + (xf), (char *)(ml)->fontx + (xt), (n)); ) \
+IFCOLOR( bcopy((char *)(ml)->color + (xf), (char *)(ml)->color + (xt), (n)); ) \
+IFCOLORX(bcopy((char *)(ml)->colorx + (xf), (char *)(ml)->colorx + (xt), (n));) \
 } while (0)
 
 #define clear_mline(ml, x, n) do {					       \
 	 bclear((char *)(ml)->image + (x), (n));			       \
 	 if ((ml)->attr != null) bzero((char *)(ml)->attr  + (x), (n));	       \
-IFFONT(  if ((ml)->font != null) bzero((char *)(ml)->font  + (x), (n));       )\
-IFCOLOR (if ((ml)->color!= null) bzero((char *)(ml)->color + (x), (n));       )\
-IFCOLORX(if ((ml)->colorx!= null) bzero((char *)(ml)->colorx + (x), (n));     )\
+IFFONT(  if ((ml)->font != null) bzero((char *)(ml)->font  + (x), (n));      ) \
+IFFONTX( if ((ml)->fontx != null) bzero((char *)(ml)->fontx + (x), (n));      ) \
+IFCOLOR( if ((ml)->color!= null) bzero((char *)(ml)->color + (x), (n));      ) \
+IFCOLORX(if ((ml)->colorx!= null) bzero((char *)(ml)->colorx + (x), (n));    ) \
 } while (0)
 
 #define cmp_mline(ml1, ml2, x) (				\
 	    (ml1)->image[x] == (ml2)->image[x]			\
 	 && (ml1)->attr[x]  == (ml2)->attr[x]			\
-IFFONT(	 && (ml1)->font[x]  == (ml2)->font[x]		       )\
-IFCOLOR( && (ml1)->color[x] == (ml2)->color[x]		       )\
-IFCOLORX(&& (ml1)->colorx[x] == (ml2)->colorx[x]	       )\
+IFFONT(	 && (ml1)->font[x]  == (ml2)->font[x]		      ) \
+IFFONTX( && (ml1)->fontx[x] == (ml2)->fontx[x]		      ) \
+IFCOLOR( && (ml1)->color[x] == (ml2)->color[x]		      ) \
+IFCOLORX(&& (ml1)->colorx[x] == (ml2)->colorx[x]	      ) \
 )
 
 #define cmp_mchar(mc1, mc2) (					\
 	    (mc1)->image == (mc2)->image			\
 	 && (mc1)->attr  == (mc2)->attr				\
-IFFONT(	 && (mc1)->font  == (mc2)->font			       )\
-IFCOLOR( && (mc1)->color == (mc2)->color		       )\
-IFCOLORX(&& (mc1)->colorx == (mc2)->colorx		       )\
+IFFONT(	 && (mc1)->font  == (mc2)->font			      ) \
+IFFONTX( && (mc1)->fontx == (mc2)->fontx		      ) \
+IFCOLOR( && (mc1)->color == (mc2)->color		      ) \
+IFCOLORX(&& (mc1)->colorx == (mc2)->colorx		      ) \
 )
 
 #define cmp_mchar_mline(mc, ml, x) (				\
 	    (mc)->image == (ml)->image[x]			\
 	 && (mc)->attr  == (ml)->attr[x]			\
-IFFONT(	 && (mc)->font  == (ml)->font[x]		       )\
-IFCOLOR( && (mc)->color == (ml)->color[x]		       )\
-IFCOLORX(&& (mc)->colorx == (ml)->colorx[x]		       )\
+IFFONT(	 && (mc)->font  == (ml)->font[x]		      ) \
+IFFONTX( && (mc)->fontx == (ml)->fontx[x]		      ) \
+IFCOLOR( && (mc)->color == (ml)->color[x]		      ) \
+IFCOLORX(&& (mc)->colorx == (ml)->colorx[x]		      ) \
 )
 
 #define copy_mchar2mline(mc, ml, x) do {			\
 	 (ml)->image[x] = (mc)->image;				\
 	 (ml)->attr[x]  = (mc)->attr;				\
-IFFONT(	 (ml)->font[x]  = (mc)->font;			       )\
-IFCOLOR( (ml)->color[x] = (mc)->color;			       )\
-IFCOLORX((ml)->colorx[x] = (mc)->colorx;		       )\
+IFFONT(	 (ml)->font[x]  = (mc)->font;			      ) \
+IFFONTX( (ml)->fontx[x] = (mc)->fontx;			      ) \
+IFCOLOR( (ml)->color[x] = (mc)->color;			      ) \
+IFCOLORX((ml)->colorx[x] = (mc)->colorx;		      ) \
 } while (0)
 
 #define copy_mline2mchar(mc, ml, x) do {			\
 	 (mc)->image = (ml)->image[x];				\
 	 (mc)->attr  = (ml)->attr[x];				\
-IFFONT(	 (mc)->font  = (ml)->font[x];			       )\
-IFCOLOR( (mc)->color = (ml)->color[x];			       )\
-IFCOLORX((mc)->colorx = (ml)->colorx[x];		       )\
-IFDWCHAR((mc)->mbcs  = 0;				       )\
+IFFONT(	 (mc)->font  = (ml)->font[x];			      ) \
+IFFONTX( (mc)->fontx = (ml)->fontx[x];			      ) \
+IFCOLOR( (mc)->color = (ml)->color[x];			      ) \
+IFCOLORX((mc)->colorx = (ml)->colorx[x];		      ) \
+IFDWCHAR((mc)->mbcs  = 0;				      ) \
 } while (0)
 
 #ifdef COLOR
@@ -139,8 +156,8 @@ IFDWCHAR((mc)->mbcs  = 0;				       )\
 #  ifdef COLORS256
 #   define rend_getbg(mc) (((mc)->color & 0xf0) >> 4 | ((mc)->attr & A_BBG ? 0x100 : 0) | ((mc)->colorx & 0xf0))
 #   define rend_setbg(mc, c) ((mc)->color = ((mc)->color & 0x0f) | (c << 4 & 0xf0), (mc)->colorx = ((mc)->colorx & 0x0f) | (c & 0xf0), (mc)->attr = ((mc)->attr | A_BBG) ^ (c & 0x100 ? 0 : A_BBG))
-#   define rend_getfg(mc) (((mc)->color & 0x0f) | ((mc)->attr & A_BFG ? 0x100 : 0) | ((mc)->colorx & 0x0f << 4))
-#   define rend_setfg(mc, c) ((mc)->color = ((mc)->color & 0xf0) | (c & 0x0f), (mc)->colorx = ((mc)->colorx & 0xf0) | (c & 0xf0 >> 4), (mc)->attr = ((mc)->attr | A_BFG) ^ (c & 0x100 ? 0 : A_BFG))
+#   define rend_getfg(mc) (((mc)->color & 0x0f) | ((mc)->attr & A_BFG ? 0x100 : 0) | (((mc)->colorx & 0x0f) << 4))
+#   define rend_setfg(mc, c) ((mc)->color = ((mc)->color & 0xf0) | (c & 0x0f), (mc)->colorx = ((mc)->colorx & 0xf0) | ((c & 0xf0) >> 4), (mc)->attr = ((mc)->attr | A_BFG) ^ (c & 0x100 ? 0 : A_BFG))
 #   define rend_setdefault(mc) ((mc)->color = (mc)->colorx = 0, (mc)->attr &= ~(A_BBG|A_BFG))
 #  else
 #   define rend_getbg(mc) (((mc)->color & 0xf0) >> 4 | ((mc)->attr & A_BBG ? 0x100 : 0))
@@ -161,3 +178,12 @@ IFDWCHAR((mc)->mbcs  = 0;				       )\
 #  define cole2i(c) ((c) ^ 9)
 # endif
 #endif
+
+enum
+{
+  REND_BELL = 0,
+  REND_MONITOR,
+  REND_SILENCE,
+  NUM_RENDS
+};
+
