@@ -1,11 +1,16 @@
-/* Copyright (c) 1993-2002
+/* Copyright (c) 2008, 2009
+ *      Juergen Weigert (jnweiger@immd4.informatik.uni-erlangen.de)
+ *      Michael Schroeder (mlschroe@immd4.informatik.uni-erlangen.de)
+ *      Micah Cowan (micah@cowan.name)
+ *      Sadrul Habib Chowdhury (sadrul@users.sourceforge.net)
+ * Copyright (c) 1993-2002, 2003, 2005, 2006, 2007
  *      Juergen Weigert (jnweiger@immd4.informatik.uni-erlangen.de)
  *      Michael Schroeder (mlschroe@immd4.informatik.uni-erlangen.de)
  * Copyright (c) 1987 Oliver Laumann
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
+ * the Free Software Foundation; either version 3, or (at your option)
  * any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -14,17 +19,15 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program (see the file COPYING); if not, write to the
- * Free Software Foundation, Inc.,
- * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
+ * along with this program (see the file COPYING); if not, see
+ * http://www.gnu.org/licenses/, or contact Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA  02111-1301  USA
  *
  ****************************************************************
  */
 
-#include "rcs.h"
-RCS_ID("$Id: mark.c,v 1.6 1994/05/31 12:32:15 mlschroe Exp $ FAU")
-
 #include <sys/types.h>
+#include <ctype.h>
 
 #include "config.h"
 #include "screen.h"
@@ -66,7 +69,7 @@ extern struct mchar mchar_so;
 int pastefont = 1;
 #endif
 
-static struct LayFuncs MarkLf =
+struct LayFuncs MarkLf =
 {
   MarkProcess,
   MarkAbort,
@@ -74,7 +77,8 @@ static struct LayFuncs MarkLf =
   DefClearLine,
   MarkRewrite,
   DefResize,
-  DefRestore
+  DefRestore,
+  0
 };
 
 int join_with_cr =  0;
@@ -137,6 +141,59 @@ int y;
   return x;
 }
 
+/*
+ * nextchar sets *xp to the num-th occurrence of the target in the line.
+ *
+ * Returns -1 if the target doesn't appear num times, 0 otherwise.
+ */
+static int
+nextchar(int *xp, int *yp, int direction, char target, int num)
+{
+  int width;  /* width of the current window. */
+  int x;      /* x coordinate of the current cursor position. */
+  int step;   /* amount to increment x (+1 or -1) */
+  int adjust; /* Final adjustment of cursor position. */
+  char *displayed_line; /* Line in which search takes place. */
+ 
+  debug("nextchar\n");
+ 
+  x = *xp;
+  step = 1;
+  adjust = 0;
+  width = fore->w_width;
+  displayed_line = (char *)WIN(*yp) -> image;
+ 
+  switch(direction) {
+    case 't':
+      adjust = -1; /* fall through */
+    case 'f':
+      step = 1;
+      break;
+    case 'T':
+      adjust = 1; /* fall through */
+    case 'F':
+      step = -1;
+      break;
+    default:
+      ASSERT(0);
+  }
+ 
+  x += step;
+ 
+  debug1("ml->image = %s\n", displayed_line);
+  debug2("num = %d, width = %d\n",num, width);
+  debug2("x = %d target = %c\n", x, target );
+ 
+  for ( ;x>=0 && x <= width; x += step) {
+    if (displayed_line[x] == target) {
+      if (--num == 0) {
+        *xp = x + adjust;
+        return 0;
+      }
+    }
+  }
+  return -1;
+}
 
 /*
  *  nextword calculates the cursor position of the num'th word.
@@ -152,6 +209,8 @@ int y;
 #define NW_MUSTMOVE	(1<<2)
 #define NW_BIG		(1<<3)
 
+
+
 static void
 nextword(xp, yp, flags, num)
 int *xp, *yp, flags, num;
@@ -224,8 +283,8 @@ char *pt;
   unsigned char *im;
   struct mline *ml;
 #ifdef FONT
-  int cf, font;
-  unsigned char *fo;
+  int cf, cfx, font;
+  unsigned char *fo, *fox;
 #endif
 
   markdata->second = 0;
@@ -273,6 +332,7 @@ char *pt;
       im = ml->image + j;
 #ifdef FONT
       fo = ml->font + j;
+      fox = ml->fontx + j;
       font = ASCII;
 #endif
       for (; j <= to; j++)
@@ -280,10 +340,11 @@ char *pt;
 	  c = (unsigned char)*im++;
 #ifdef FONT
 	  cf = (unsigned char)*fo++;
+	  cfx = (unsigned char)*fox++;
 # ifdef UTF8
 	  if (fore->w_encoding == UTF8)
 	    {
-	      c |= cf << 8;
+	      c |= cf << 8 | cfx << 16;
 	      if (c == UCS_HIDDEN)
 		continue;
 	      c = ToUtf8_comb(pt, c);
@@ -445,6 +506,7 @@ MarkRoutine()
   if (InitOverlayPage(sizeof(*markdata), &MarkLf, 1))
     return;
   flayer->l_encoding = fore->w_encoding;
+  flayer->l_mode = 1;
   markdata = (struct markdata *)flayer->l_data;
   markdata->md_user = D_user;	/* XXX: Correct? */
   markdata->md_window = fore;
@@ -503,21 +565,24 @@ int *inlenp;
   in_mark = 1;
   while (in_mark && (inlen /* || extrap */))
     {
-/*
-      if (extrap)
+      unsigned char ch = (unsigned char )*pt++;
+      inlen--;
+      if (flayer->l_mouseevent.start)
 	{
-	  od = *extrap++;
-	  if (*extrap == 0)
-	    extrap = 0;
-	}
-      else
-*/
-	{
-          od = mark_key_tab[(int)(unsigned char)*pt++];
-          inlen--;
+	  int r = LayProcessMouse(flayer, ch);
+	  if (r == -1)
+	    LayProcessMouseSwitch(flayer, 0);
+	  else
+	    {
+	      if (r)
+		ch = 0222;
+	      else
+		continue;
+	    }
 	}
+      od = mark_key_tab[(int)ch];
       rep_cnt = markdata->rep_cnt;
-      if (od >= '0' && od <= '9')
+      if (od >= '0' && od <= '9' && !markdata->f_cmd.flag)
         {
 	  if (rep_cnt < 1001 && (od != '0' || rep_cnt != 0))
 	    {
@@ -537,8 +602,51 @@ int *inlenp;
 	}
       cx = markdata->cx;
       cy = markdata->cy;
+
+      if (markdata -> f_cmd.flag) {
+        debug2("searching for %c:%d\n",od,rep_cnt);
+        markdata->f_cmd.flag = 0;
+        markdata->rep_cnt = 0;
+
+	if (isgraph (od)) {
+	  markdata->f_cmd.target = od;
+	  rep_cnt = (rep_cnt) ? rep_cnt : 1;
+	  nextchar(&cx, &cy, markdata->f_cmd.direction, od, rep_cnt );
+	  revto(cx, cy);
+	  continue;
+	}
+      }
+
+processchar:
       switch (od)
-	{
+        {
+	case 'f': /* fall through */
+	case 'F': /* fall through */
+	case 't': /* fall through */
+	case 'T': /* fall through */
+	  /*
+	   * Set f_cmd to do a search on the next key stroke.
+	   * If we break, rep_cnt will be reset, so we
+	   * continue instead. It might be cleaner to
+	   * store the rep_count in f_cmd and
+	   * break here so later followon code will be
+	   * hit.
+	   */
+	  markdata->f_cmd.flag = 1;
+	  markdata->f_cmd.direction = od;
+	  debug("entering char search\n");
+	  continue;
+	case ';':
+	case ',':
+	  if (!markdata->f_cmd.target)
+	    break;
+	  if (!rep_cnt)
+	    rep_cnt = 1;
+	  nextchar(&cx, &cy,
+	      od == ';' ? markdata->f_cmd.direction : (markdata->f_cmd.direction ^ 0x20),
+	      markdata->f_cmd.target, rep_cnt );
+	  revto(cx, cy);
+	  break;
 	case 'o':
 	case 'x':
 	  if (!markdata->second)
@@ -666,7 +774,7 @@ int *inlenp;
             LGotoPos(flayer, cx, W2D(cy));
 	  break;
 	case '@':
-	  /* it may be usefull to have a key that does nothing */
+	  /* it may be useful to have a key that does nothing */
 	  break;
 	case '%':
 	  rep_cnt--;
@@ -797,6 +905,11 @@ int *inlenp;
 	case 'n':
 	  Search(0);
 	  break;
+	case 'N':
+	  markdata->isdir = -markdata->isdir;
+	  Search(0);
+	  markdata->isdir = -markdata->isdir;
+	  break;
 	case 'y':
 	case 'Y':
 	  if (markdata->second == 0)
@@ -926,6 +1039,7 @@ int *inlenp;
 		  LAY_CALL_UP(LRefreshAll(flayer, 0));
 		}
 	      ExitOverlayPage();
+	      WindowChanged(fore, 'P');
 	      if (append_mode)
 		LMsg(0, "Appended %d characters to buffer",
 		    newcopylen);
@@ -936,6 +1050,39 @@ int *inlenp;
 	      in_mark = 0;
 	      break;
 	    }
+
+	case 0222:
+	  if (flayer->l_mouseevent.start)
+	    {
+	      int button = flayer->l_mouseevent.buffer[0];
+	      if (button == 'a')
+		{
+		  /* Scroll down */
+		  od = 'j';
+		}
+	      else if (button == '`')
+		{
+		  /* Scroll up */
+		  od = 'k';
+		}
+	      else if (button == ' ')
+		{
+		  /* Left click */
+		  cx = flayer->l_mouseevent.buffer[1];
+		  cy = D2W(flayer->l_mouseevent.buffer[2]);
+		  revto(cx, cy);
+		  od = ' ';
+		}
+	      else
+		od = 0;
+	      LayProcessMouseSwitch(flayer, 0);
+	      if (od)
+		goto processchar;
+	    }
+	  else
+	    LayProcessMouseSwitch(flayer, 1);
+	  break;
+
 	default:
 	  MarkAbort();
 	  LMsg(0, "Copy mode aborted");
@@ -1012,6 +1159,8 @@ int tx, ty, line;
 
   if (markdata->second == 0)
     {
+      flayer->l_x = tx;
+      flayer->l_y = W2D(ty);
       LGotoPos(flayer, tx, W2D(ty));
       return;
     }
@@ -1087,7 +1236,10 @@ int tx, ty, line;
 	      mc = mchar_so;
 #ifdef FONT
 	      if (pastefont)
-		mc.font = ml->font[x];
+		{
+		  mc.font = ml->font[x];
+		  mc.fontx = ml->fontx[x];
+		}
 #endif
 	      mc.image = ml->image[x];
 	    }
@@ -1108,6 +1260,8 @@ int tx, ty, line;
 #endif
 	}
     }
+  flayer->l_x = tx;
+  flayer->l_y = W2D(ty);
   LGotoPos(flayer, tx, W2D(ty));
 }
 
@@ -1135,6 +1289,7 @@ MarkAbort()
       rem(markdata->x1, markdata->y1, markdata->cx, markdata->cy, redisp, (char *)0, yend);
     }
   ExitOverlayPage();
+  WindowChanged(fore, 'P');
 }
 
 
@@ -1204,7 +1359,10 @@ int isblank;
 	break;
 #ifdef FONT
       if (pastefont)
-	mchar_marked.font = ml->font[x];
+	{
+	  mchar_marked.font = ml->font[x];
+	  mchar_marked.fontx = ml->fontx[x];
+	}
 #endif
       mchar_marked.image = ml->image[x];
 #ifdef DW_CHARS
@@ -1283,7 +1441,10 @@ struct mchar *rend;
         {
 #ifdef FONT
 	  if (pastefont)
-	    mchar_marked.font = ml->font[x];
+	    {
+	      mchar_marked.font = ml->font[x];
+	      mchar_marked.fontx = ml->fontx[x];
+	    }
 #endif
 	  rend->image = mchar_marked.image;
 	  if (!cmp_mchar(rend, &mchar_marked))
@@ -1341,14 +1502,6 @@ int n;
   return n;
 }
 
-int
-InMark()
-{
-  if (flayer && flayer->l_layfn == &MarkLf)
-    return 1;
-  return 0;
-}
-
 void
 MakePaster(pa, buf, len, bufiscopy)
 struct paster *pa;
@@ -1373,6 +1526,7 @@ struct paster *pa;
     free(pa->pa_pastebuf);
   pa->pa_pastebuf = 0;
   pa->pa_pasteptr = 0;
+  pa->pa_pastelen = 0;
   pa->pa_pastelayer = 0;
   evdeq(&pa->pa_slowev);
 }
